/* 
 * Sample program for use with Product          
 *  ProgIds: 5724-J06 5724-J05 5724-J04 5697-J09 5655-M74 5655-M75 5648-C63 
 *  (C) Copyright IBM Corporation 2005.                      
 * All Rights Reserved * Licensed Materials - Property of IBM 
 * 
 * This sample program is provided AS IS and may be used, executed, 
 * copied and modified without royalty payment by customer 
 * 
 * (a) for its own instruction and study, 
 * (b) in order to develop applications designed to run with an IBM 
 *     WebSphere product, either for customer's own internal use or for 
 *     redistribution by customer, as part of such an application, in 
 *     customer's own products. 
 */ 

/* Includes */
/* <malloc.h> is not available on /390 */
#ifndef __MVS__
  #include <malloc.h>
#endif
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>
#include <NumComputeNode.h>
#include <Common.h>

/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */
/*  Node definition - NumComputeNode                                          */
/*                                                                            */
/******************************************************************************/
void defineNumComputeNode(
  void* factoryObject
){
  static char* functionName = (char *)"defineNumComputeNode()";
  static CNI_VFT vftable = {CNI_VFT_DEFAULT};
  int            rc = 0;

  /* Setup function table with pointers to node implementation functions */
  vftable.iFpCreateNodeContext = _NumCompute_createNodeContext;
  vftable.iFpDeleteNodeContext = _deleteNodeContext;
  vftable.iFpGetAttributeName  = _getAttributeName;
  vftable.iFpSetAttribute      = _setAttribute;
  vftable.iFpGetAttribute      = _getAttribute;
  vftable.iFpEvaluate          = _NumCompute_evaluate;

  /* Define a node type supported by our factory. If any errors/exceptions    */
  /* occur during the execution of this utility function, then as we have not */
  /* supplied the returnCode argument, the exception will bypass the plugin   */
  /* and be directly handled by the broker.                                   */
  cniDefineNodeClass(&rc, factoryObject, (CciChar*)constNumComputeNode, &vftable);

  return;
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                cniCreateNodeContext() */
/*                                                                            */
/******************************************************************************/
CciContext* _NumCompute_createNodeContext(
  CciFactory* factoryObject,
  CciChar*    nodeName,
  CciNode*    nodeObject
){
  static char* functionName = (char *)"_NumCompute_createNodeContext()";
  NODE_CONTEXT_ST* p;
  CciChar          buffer[256];

  /* Allocate a pointer to the local context */
  p = (NODE_CONTEXT_ST *)malloc(sizeof(NODE_CONTEXT_ST));

  if (p) {

    /* Clear the context area */
    memset(p, 0, sizeof(NODE_CONTEXT_ST));

    /* Save our node object pointer in our context */
    p->nodeObject = nodeObject;

    /* Save our node name */
    CciCharNCpy((CciChar*)&p->nodeName, nodeName, MAX_NODE_NAME_LEN);

    /* Create attributes and set default values */
    {
      const CciChar* ucsAttr = CciString("nodeTraceSetting", BIP_DEF_COMP_CCSID) ;
      insAttrTblEntry(p, (CciChar*)ucsAttr, CNI_TYPE_INTEGER);
      _setAttribute(p, (CciChar*)ucsAttr, (CciChar*)constZero);
      free((void *)ucsAttr) ;
    }
    {
      const CciChar* ucsAttr = CciString("nodeTraceOutfile", BIP_DEF_COMP_CCSID) ;
      insAttrTblEntry(p, (CciChar*)ucsAttr , CNI_TYPE_STRING);
      _setAttribute( p, (CciChar*)ucsAttr,
                    (CciChar*)constNumComputeTraceLocation);
      free((void *)ucsAttr) ;
    }

    /* Create terminals */
    {
      const CciChar* ucsIn = CciString("In", BIP_DEF_COMP_CCSID) ;
      insInputTerminalListEntry(p, (CciChar*)ucsIn);
      free((void *)ucsIn) ;
    }
    {
      const CciChar* ucsOut = CciString("Out", BIP_DEF_COMP_CCSID) ;
      insOutputTerminalListEntry(p, (CciChar*)ucsOut);
      free((void *)ucsOut) ;
    }
    {
      const CciChar* ucsFailure = CciString("Failure", BIP_DEF_COMP_CCSID) ;
      insOutputTerminalListEntry(p, (CciChar*)ucsFailure);
      free((void *)ucsFailure) ;
    }

    /* Get the value of this attribute */
    _getAttribute(p, (CciChar*)constNodeTraceOutfile, buffer, sizeof(buffer));

    /* Initialise default trace setting */
    p->trace = 1; 

    /* If tracing, open the file */
    if (p->trace) {
      const char * filename = mbString(buffer, BIP_DEF_COMP_CCSID);
      p->tracefile = fopen(filename, "w+");
      free((void *)filename);
    }

  }
  else {
    /* Error: Unable to allocate memory */
  }

  /* Return the pointer to the local context */
  return((CciContext*)p);
}


/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                         cniEvaluate() */
/*                                                                            */
/******************************************************************************/
void _NumCompute_evaluate(
  CciContext* context,
  CciMessage* destinationList,
  CciMessage* exceptionList,
  CciMessage* message
){
  static char* functionName = (char *)"_NumCompute_evaluate()";
  CciMessage*      outMsg;
  CciTerminal*     terminalObject;
  CciElement*      inRootElement;
  CciElement*      outRootElement;
  CciElement*      bodyChild;
  CciElement*      message2Element;
  CciElement*      inputPairElement;
  CciElement*      num1Element;
  CciElement*      num2Element;
  CciElement*      resultsElement;
  CciElement*      num1ResultsElement;
  CciElement*      num2ResultsElement;
  CciElement*      num3ResultsElement;
  CciElement*      resultResultsElement;
  int              retvalue;
  int              rc = 0;
  int              num1;
  int              num2;
  int              num3;
  int              numResult;
  CCI_EXCEPTION_ST exception_st = {CCI_EXCEPTION_ST_DEFAULT};
  CciChar          elementName[2304];
  CciChar          elementValue[512];

  /* Get the root element of the input message */
  inRootElement = cniRootElement(&rc, message);
  /*CCI_CHECK_RC();*/
  checkRC(rc);

  /* Create a new message for output */
  outMsg = cniCreateMessage(&rc, cniGetMessageContext(&rc, message));
  /*CCI_CHECK_RC();*/
  checkRC(rc);

  /* Get the root element of the output message */
  outRootElement = cniRootElement(&rc, outMsg);
  /*CCI_CHECK_RC();*/
  checkRC(rc);

  /* Copy the contents of the input message to the output message */
  cniCopyElementTree(&rc, inRootElement, outRootElement);
  /*CCI_CHECK_RC();*/
  checkRC(rc);

  /* Get the last child of root (ie the body) */
  bodyChild = cniLastChild(&rc, outRootElement);
  /*CCI_CHECK_RC();*/
  checkRC(rc);

  /* and it's name */
  cniElementName(&rc, bodyChild, (CciChar*)&elementName, sizeof(elementName));
  /*CCI_CHECK_RC();*/
  checkRC(rc);



  /* Navigate to the root of the XML message, that is the <Message2> element */
  message2Element = cniFirstChild(&rc, bodyChild);
  /*CCI_CHECK_RC();*/
  checkRC(rc);



  /* Navigate to the first child, that is the <InputPair> element */
  inputPairElement = cniFirstChild(&rc, message2Element);
  /*CCI_CHECK_RC();*/
  checkRC(rc);



  /* Navigate to the next child, that is the <Num1> element */
  num1Element = cniFirstChild(&rc, inputPairElement);
  /*CCI_CHECK_RC();*/
  checkRC(rc);

  /* Get the integer value of <Num1> element */
  cniElementCharacterValue(&rc, num1Element, (CciChar*)&elementValue, sizeof(elementValue));
  /*CCI_CHECK_RC();*/
  checkRC(rc);

  /* Convert from CciChar* to int */
  num1 = atoi(mbString(elementValue, BIP_DEF_COMP_CCSID));



  /* Navigate to the next sibling, that is the <Num2> element */
  num2Element = cniNextSibling(&rc, num1Element);
  /*CCI_CHECK_RC();*/
  checkRC(rc);

  /* Get the integer value of <Num2> element */
  cniElementCharacterValue(&rc, num2Element, (CciChar*)&elementValue, sizeof(elementValue));
  /*CCI_CHECK_RC();*/
  checkRC(rc);

  /* Convert from CciChar* to int */
  num2 = atoi(mbString(elementValue, BIP_DEF_COMP_CCSID));



  /* Perform calculations on input data as part of the test */
  num3 = num1 * num2;
  num2 = num1 * num3;
  numResult = num1 * num2 * num3;



  /* Add <Results> element as sibling of <InputPair> */
  resultsElement = cniCreateElementAfter(&rc, inputPairElement); 
  /*CCI_CHECK_RC();*/
  checkRC(rc);
  cniSetElementName(&rc, resultsElement, CciString("Results", BIP_DEF_COMP_CCSID));
  /*CCI_CHECK_RC();*/
  checkRC(rc);



  /* Add <Num1> element as first child of <Results> element, and its value */
  num1ResultsElement = cniCreateElementAsFirstChild(&rc, resultsElement); 
  /*CCI_CHECK_RC();*/
  checkRC(rc);
  cniSetElementName(&rc, num1ResultsElement, CciString("Num1", BIP_DEF_COMP_CCSID));
  /*CCI_CHECK_RC();*/
  checkRC(rc);
  cniSetElementIntegerValue(&rc, num1ResultsElement, num1);
  /*CCI_CHECK_RC();*/
  checkRC(rc);



  /* Add <Num2> element as sibling of <Num1> element, and its value */
  num2ResultsElement = cniCreateElementAfter(&rc, num1ResultsElement); 
  /*CCI_CHECK_RC();*/
  checkRC(rc);
  cniSetElementName(&rc, num2ResultsElement, CciString("Num2", BIP_DEF_COMP_CCSID));
  /*CCI_CHECK_RC();*/
  checkRC(rc);
  cniSetElementIntegerValue(&rc, num2ResultsElement, num2);
  /*CCI_CHECK_RC();*/
  checkRC(rc);



  /* Add <Num3> element as sibling of <Num2> element, and its value */
  num3ResultsElement = cniCreateElementAfter(&rc, num2ResultsElement); 
  /*CCI_CHECK_RC();*/
  checkRC(rc);
  cniSetElementName(&rc, num3ResultsElement, CciString("Num3", BIP_DEF_COMP_CCSID));
  /*CCI_CHECK_RC();*/
  checkRC(rc);
  cniSetElementIntegerValue(&rc, num3ResultsElement, num3);
  /*CCI_CHECK_RC();*/
  checkRC(rc);



  /* Add <Result> element as sibling of <Num3> element, and its value */
  resultResultsElement = cniCreateElementAfter(&rc, num3ResultsElement); 
  /*CCI_CHECK_RC();*/
  checkRC(rc);
  cniSetElementName(&rc, resultResultsElement, CciString("Result", BIP_DEF_COMP_CCSID));
  /*CCI_CHECK_RC();*/
  checkRC(rc);
  cniSetElementIntegerValue(&rc, resultResultsElement, numResult);
  /*CCI_CHECK_RC();*/
  checkRC(rc);
  


  /* Get handle of output terminal */
  terminalObject = getOutputTerminalHandle( (NODE_CONTEXT_ST *)context,
                                            (CciChar*)constOut);

  /* If the terminal exists and is attached, propagate to it */
  if (terminalObject) {
    if (cniIsTerminalAttached(&rc, terminalObject)) {
      /* As this is a new, and changed message, it should be finalized... */
      cniFinalize(&rc, outMsg, CCI_FINALIZE_NONE);
      retvalue = cniPropagate(&rc, terminalObject, destinationList, exceptionList, outMsg);
      if (retvalue == CCI_FAILURE) {
        if (rc == CCI_EXCEPTION) {
          /* Get details of the exception */
          memset(&exception_st, 0, sizeof(exception_st));
          cciGetLastExceptionData(&rc, &exception_st);

          /* Any local error handling may go here */

          /* Ensure message is deleted prior to return/throw */
          cniDeleteMessage(0, outMsg);

          /* We must "rethrow" the exception; note this does not return */
          cciRethrowLastException(&rc);
        }
        else {

          /* Some other error...the plugin might choose to log it using the CciLog() */
          /* utility function                                                        */

        }
      }
      else {
      }
    }
  }
  else {
    /* Terminal did not exist...severe internal error. The plugin may wish to */
    /* log an error here using the cciLog() utility function.                 */
  }

  /* Delete the message we created now we have finished with it */
  cniDeleteMessage(0, outMsg);

  return;
}

